
var Relax = function()
{
	var relax = {};
	
	relax.VERSION = '1.0';

	return relax;
}();


var Wizard = function()
{
	var wizard = {};
	var internal = {};
	var debug = 1;
	
	var numLiveAsyncRequests = 0;
	var hasFatalError = false;
	
	/**
		Submit an ansynchronous request for JSON data. 'GET' method
		is used if 'method' parameter is not set.
		The 'callback' should be a method accepting a single
		JSON parameter object. If the 'status' of the response
		is not 'ok' this method will call setFatalError and
		without calling the callback method.
		
		The 'errorCallback' is an optional function parameter that should
		take two parameters. The first is an Error object and the second is
		the JSON response. The seconds parameter only has a value if the error 
		happens after parsing the response and is null otherwise.
	*/
	wizard.asyncJsonRequest = function(url, callback, method, postdata, errorCallback)
	{
		if (debug)
		{
			App.debug((method || 'GET') + ': ' + url);
			if (postdata) App.debug(postdata);
		}
		var request = Ajax.getXmlHttpRequest();
		request.jsonCallback = callback;
		request.errorCallback = errorCallback;
		request.open(method || 'GET', url, true);
		// Firefox only adds request header with needed "boundary" info, if request header is not set explicitly
		if (postdata && !(postdata instanceof FormData)) 
		{
			request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
		}
		Ajax.setReadyStateHandler(request, internal.asyncCallbackWrapper, internal.asyncCallbackWrapper);
		numLiveAsyncRequests++;
		request.send(postdata || null);
	}
	
	/**
		Callback wrapper for ansynchronous requests.
	*/
	internal.asyncCallbackWrapper = function(request)
	{
		numLiveAsyncRequests--;
		if (debug) App.debug(request.responseText);
		if (!hasFatalError) Wizard.hideWizardStatus();

		var response = null;
		var messages = null;
		try
		{
			if (request.status != 200)
			{
				throw new Error(request.responseText);
			}
			else
			{
				if (request.getResponseHeader('Content-Type').indexOf('json') != -1)
				{
					response = JSON.parse(request.responseText);
					messages = response.messages;
					if (response.status != 'ok')
					{
						throw new Error(response.message || response.stacktrace);
					}
				}
				else
				{
					response = request.responseText;
				}
				if (request.jsonCallback && request.jsonCallback.call)
				{
					request.jsonCallback.call(null, response);
				}
			}
		}
		catch (err)
		{
			if (request.errorCallback && request.errorCallback.call)
			{
				request.errorCallback.call(null, err, response);
			}
			else
			{
				Wizard.setFatalError(err, messages);
			}
		}
	}

	
	/**
		A fatal error has occurred and the wizard will not be able to continue.
		This method will display the error message, disable all form input fields
		and buttons (except the 'Restart' button).
	*/
	wizard.setFatalError = function(err, messages)
	{
		hasFatalError = true;
		var msg = err;
		if (err.toString)
		{
			App.debug(err.toString());
			msg = Strings.encodeTags(err.toString());
			if (err.stack && !messages)
			{
				App.debug(err.stack);
				msg += '<br><pre>'+Strings.encodeTags(err.stack)+'</pre>';
			}
		}
		else
		{
			App.debug(msg);
		}
		
		if (messages) 
		{
			msg = '<div class="messagecontainer error">'+msg+'</div>'+internal.generateMessageList(messages).html;
			wizard.setWizardStatus('success', msg);
		}
		else
		{
			wizard.setWizardStatus('messagecontainer error', msg || 'Unexpected failure!');
		}
		
		Doc.addClass('wizard', 'disabled');
		internal.disableAllFormElements('wizard', true);
		Doc.show('gorestart');
	}

	
	/**
		Display a 'loading' global wizard-status message.
		The 'progress' parameter is optional. If given,
		it should be the name of a progress reporter.
		This activates the display of a progress bar which
		Relax will update at regular intervals in the background.
		See SessionServlet (cmd=GetProgress) for more information.
	*/
	wizard.showLoadingAnimation = function(msg, progress)
	{
		wizard.setWizardStatus('loading', msg  || 'Working, please wait...');
	}
	
	wizard.showStatusMessage = function(msg)
	{
		App.debug(msg);
		wizard.setWizardStatus('messagecontainer note', msg);
	}

	/**
		Hide the current global wizard-status message.
	*/
	wizard.hideWizardStatus = function()
	{
		Doc.hide('wizard-status');
		Doc.hide('wizard-progress');
	}
		
	/**
		Initialize the wizard:
		
		* Check that a project is active
	*/
	internal.initWizard = function()
	{
		var path = document.getElementsByClassName('path');
		if (path && path.length)
		{
			var relaxVersion = document.createElement('span');
			relaxVersion.id = 'relax-version';
			relaxVersion.innerHTML = '[Relax '+Relax.VERSION+']';
			path[0].appendChild(relaxVersion);
		}
	}
		
	/**
		Display a global wizard-status message. This may for example be
		an error message, a work-in-progress indicator, etc.
	*/
	wizard.setWizardStatus = function(className, msg)
	{
		var status = Doc.element('wizard-status');
		status.className = className;
		status.innerHTML = msg;
		Doc.show(status);
	}

	// Disable all form elements inside the given container
	internal.disableAllFormElements = function(container, hideButtons)
	{
		container = Doc.element(container);
		var input = container.getElementsByTagName('input');
		for (var i = 0; i < input.length; i++)
		{
			input[i].disabled = true;
		}
		var select = container.getElementsByTagName('select');
		for (var i = 0; i < select.length; i++)
		{
			select[i].disabled = true;
		}
		var text = container.getElementsByTagName('textarea');
		for (var i = 0; i < text.length; i++)
		{
			text[i].disabled = true;
		}
		if (hideButtons)
		{
			internal.hideButtons(container);
		}
	}

	// Hide all 'basicbuttons':s in the given container
	internal.hideButtons = function(container)
	{
		container = Doc.element(container);
		var buttons = container.getElementsByClassName('basicbutton');
		for (var i = 0; i < buttons.length; i++)
		{
			Doc.hide(buttons[i]);
		}
	}

	Doc.addFinalizer(internal.initWizard);
	return wizard;
}();

/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(item)' method
	to evaluate all criterias against a single item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given item property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.eq;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Adds a text criteria checking if the given item property to
	contains a static value (f2).
*/
Filter.prototype.addContainsCriteria = function(itemProp, f2)
{
	if (f2 != '') 
	{
		var cmp;
		if (f2 == '=')
		{
			cmp = Comparator.eq;
			f2 = null;
		}
		else if (f2 == '<>')
		{
			cmp = Comparator.neq;
			f2 = null;
		}
		else
		{
			cmp = Comparator.contains;
		}
		this.criterias[this.criterias.length] = function(item) { return cmp(itemProp(item), f2) };
	}
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(item)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](item)) return false;
	}
	return true;
}


var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are different
	cmp.neq = function(v1, v2)
	{
		return v1 != v2;
	}
	
	// Check if v2 is contained in v1
	cmp.contains = function(v1, v2)
	{
		return v1 != null && v2 != null && v1.indexOf(v2) >= 0;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 starts with the same string as v2
	cmp.startsWith = function(v1, v2)
	{
		return v1 != null && v1.indexOf(v2) == 0;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 != null && v2 != null && v1 <= v2;
	}
	
	return cmp;
}();



